@@ -2,7 +2,7 @@
 
     PCMCIA Card Manager daemon
 
-    cardmgr.c 1.114 1998/08/09 00:15:41
+    cardmgr.c 1.121 1999/02/06 07:16:17
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.0 (the "License"); you may not use this file
@@ -48,17 +48,6 @@
 #include <pcmcia/cistpl.h>
 #include <pcmcia/ds.h>
 
-#define VERSION(v,p,s) (((v)<<16)+(p<<8)+s)
-#if (LINUX_VERSION_CODE > VERSION(1,3,0))
-#define GET_SCSI_INFO
-#include <linux/major.h>
-#if (LINUX_VERSION_CODE > VERSION(1,3,97))
-#include <scsi/scsi.h>
-#else
-#include <linux/scsi.h>
-#endif
-#endif
-
 #include "cardmgr.h"
 
 /*====================================================================*/
@@ -159,10 +148,6 @@ static int lookup_dev(char *name)
 	return -ENODEV;
 }
 
-#endif /* __linux__ */
-
-/*====================================================================*/
-
 int open_dev(dev_t dev, int mode)
 {
     char *fn;
@@ -176,6 +161,8 @@ int open_dev(dev_t dev, int mode)
     return fd;
 }
 
+#endif /* __linux__ */
+
 int open_sock(int sock, int mode)
 {
 #ifdef __linux__
@@ -198,21 +185,29 @@ int open_sock(int sock, int mode)
     
 ======================================================================*/
 
-#ifdef GET_SCSI_INFO
+#ifdef __linux__
+
+#include <linux/major.h>
+#include <scsi/scsi.h>
+#define VERSION(v,p,s) (((v)<<16)+(p<<8)+s)
+#if (LINUX_VERSION_CODE < VERSION(2,1,126))
+#define SCSI_DISK0_MAJOR SCSI_DISK_MAJOR
+#endif
+
 static int xlate_scsi_name(bind_info_t *bind)
 {
     int i, fd, mode, minor;
     u_long arg[2], id1, id2;
 
     id1 = strtol(bind->name+3, NULL, 16);
-    if ((bind->major == SCSI_DISK_MAJOR) ||
+    if ((bind->major == SCSI_DISK0_MAJOR) ||
 	(bind->major == SCSI_CDROM_MAJOR))
 	mode = S_IREAD|S_IFBLK;
     else
 	mode = S_IREAD|S_IFCHR;
     
     for (i = 0; i < 16; i++) {
-	minor = (bind->major == SCSI_DISK_MAJOR) ? (i<<4) : i;
+	minor = (bind->major == SCSI_DISK0_MAJOR) ? (i<<4) : i;
 	fd = open_dev((bind->major<<8)+minor, mode);
 	if (fd < 0)
 	    continue;
@@ -222,7 +217,7 @@ static int xlate_scsi_name(bind_info_t *bind)
 	    if (id1 == id2) {
 		close(fd);
 		switch (bind->major) {
-		case SCSI_DISK_MAJOR:
+		case SCSI_DISK0_MAJOR:
 		case SCSI_GENERIC_MAJOR:
 		    sprintf(bind->name+2, "%c", 'a'+i); break;
 		case SCSI_CDROM_MAJOR:
@@ -271,7 +266,7 @@ static void beep(unsigned int ms, unsigned int freq)
 #ifdef __BEOS__
 static void beep(unsigned int ms, unsigned int freq)
 {
-    system("/bin/beep");
+    if (!be_quiet) system("/bin/beep");
 }
 #endif
 
@@ -398,6 +393,7 @@ static card_info_t *lookup_card(int ns)
     cistpl_vers_1_t *vers = NULL;
     cistpl_manfid_t manfid = { 0, 0 };
     cistpl_funcid_t funcid = { 0xff, 0xff };
+    cs_status_t status;
     int i, ret, match;
     int has_cis = 0;
 
@@ -484,8 +480,11 @@ static card_info_t *lookup_card(int ns)
 	    return card;
 	}
     }
-    
-    if (vers || (!blank_card)) {
+
+    status.Function = 0;
+    if ((ioctl(s->fd, DS_GET_STATUS, &status) != 0) ||
+	(status.CardState & CS_EVENT_CB_DETECT) ||
+	manfid.manf || manfid.card || vers || !blank_card) {
 	syslog(LOG_INFO, "unsupported card in socket %d", ns);
 	if (one_pass) return NULL;
 	beep(BEEP_TIME, BEEP_ERR);
@@ -898,7 +897,7 @@ static void do_insert(int sn)
 	tail = &s->bind[i];
 	while (ret == 0) {
 	    bind_info_t *old;
-#ifdef GET_SCSI_INFO
+#ifdef __linux__
 	    if ((strlen(bind->name) > 3) &&
 		(bind->name[2] == '#'))
 		xlate_scsi_name(bind);
@@ -1164,23 +1163,17 @@ static void handle_signal(void)
     case SIGTERM:
     case SIGINT:
 	for (i = 0; i < sockets; i++)
-	    if (socket[i].state & SOCKET_PRESENT)
-		do_remove(i);
+	    if ((socket[i].state & SOCKET_PRESENT) &&
+		(do_check(i) == 0)) do_remove(i);
 	free_resources();
 	exit(0);
 	break;
     case SIGHUP:
-	for (i = 0; i < sockets; i++)
-	    if (socket[i].state & SOCKET_PRESENT)
-		do_remove(i);
 	free_resources();
 	free_config();
 	syslog(LOG_INFO, "re-loading config file");
 	load_config();
 	adjust_resources();
-	for (i = 0; i < sockets; i++)
-	    if (socket[i].state & SOCKET_PRESENT)
-		do_insert(i);
 	break;
 #ifdef SIGPWR
     case SIGPWR:
@@ -1297,13 +1290,17 @@ int main(int argc, char *argv[])
 
 #ifdef __linux__
     if (modpath == NULL) {
-	struct utsname utsname;
-	if (uname(&utsname) != 0) {
-	    syslog(LOG_INFO, "uname(): %m");
-	    exit(EXIT_FAILURE);
+	if (access("/lib/modules/preferred", X_OK) == 0)
+	    modpath = "/lib/modules/preferred";
+	else {
+	    struct utsname utsname;
+	    if (uname(&utsname) != 0) {
+		syslog(LOG_INFO, "uname(): %m");
+		exit(EXIT_FAILURE);
+	    }
+	    modpath = (char *)malloc(32);
+	    sprintf(modpath, "/lib/modules/%s", utsname.release);
 	}
-	modpath = (char *)malloc(32);
-	sprintf(modpath, "/lib/modules/%s", utsname.release);
     }
     if (access(modpath, X_OK) != 0) {
 	syslog(LOG_INFO, "cannot access %s: %m", modpath);
