@@ -1,4 +1,20 @@
-/* cardmgr.h $Revision: 1.24 $ $Date: 1997/12/24 17:28:02 $ (David Hinds) */
+/*
+ * cardmgr.h 1.28 1998/08/03 17:14:06
+ *
+ * The contents of this file are subject to the Mozilla Public License
+ * Version 1.0 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License
+ * at http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS"
+ * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
+ * the License for the specific language governing rights and
+ * limitations under the License. 
+ *
+ * The initial developer of the original code is David A. Hinds
+ * <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
+ * are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
+ */
 
 #define MAX_SOCKS	8
 #define MAX_FUNCTIONS	4
@@ -54,6 +70,7 @@ typedef struct card_info_t {
     int			functions;
     device_info_t	*device[MAX_FUNCTIONS];
     int			dev_fn[MAX_FUNCTIONS];
+    char		*cis_file;
     struct card_info_t	*next;
 } card_info_t;
 
@@ -63,7 +80,7 @@ typedef struct mtd_ident_t {
 	JEDEC_MTD=1, DTYPE_MTD, DEFAULT_MTD
     } mtd_type;
     int			dtype, jedec_mfr, jedec_info;
-    char		*module;
+    char		*module, *opts;
     struct mtd_ident_t	*next;
 } mtd_ident_t;
     
