@@ -2,7 +2,7 @@
 
     PCMCIA controller probe
 
-    probe.c 1.34 1998/08/01 19:01:14
+    probe.c 1.39 1999/01/14 07:33:11
 
     The contents of this file are subject to the Mozilla Public
     License Version 1.0 (the "License"); you may not use this file
@@ -58,16 +58,27 @@ pci_id_t pci_id[] = {
     { 0x1013, 0x6729, "Cirrus Logic CL 6729", "Cirrus PD6729" },
     { 0x1013, 0x1110, "Cirrus Logic PD 6832", "Cirrus PD6832" },
     { 0x10b3, 0xb106, "SMC 34C90", "SMC 34C90" },
+    { 0x1180, 0x0465, "Ricoh RL5C465", "Ricoh RL5C465" },
     { 0x1180, 0x0466, "Ricoh RL5C466", "Ricoh RL5C466" },
+    { 0x1180, 0x0475, "Ricoh RL5C475", "Ricoh RL5C475" },
+    { 0x1180, 0x0476, "Ricoh RL5C476", "Ricoh RL5C476" },
+    { 0x1180, 0x0478, "Ricoh RL5C478", "Ricoh RL5C478" },
     { 0x104c, 0xac12, "Texas Instruments PCI1130", "TI 1130" },
     { 0x104c, 0xac13, "Texas Instruments PCI1031", "TI 1031" },
     { 0x104c, 0xac15, "Texas Instruments PCI1131", "TI 1131" },
     { 0x104c, 0xac16, "Texas Instruments PCI1250", "TI 1250A" },
     { 0x104c, 0xac17, "Texas Instruments PCI1220", "TI 1220" },
+    { 0x104c, 0xac19, "Texas Instruments PCI1221", "TI 1221" },
+    { 0x104c, 0xac1a, "Texas Instruments PCI1210", "TI 1210" },
+    { 0x104c, 0xac1d, "Texas Instruments PCI1251A", "TI 1251A" },
+    { 0x104c, 0xac1f, "Texas Instruments PCI1251B", "TI 1251B" },
+    { 0x104c, 0xac1b, "Texas Instruments PCI1450", "TI 1450" },
     { 0x1217, 0x6729, "O2 Micro 6729", "O2Micro OZ6729" },
     { 0x1217, 0x673a, "O2 Micro 6730", "O2Micro OZ6730" },
     { 0x1217, 0x6832, "O2 Micro 6832", "O2Micro OZ6832" },
-    { 0x1179, 0x060a, "Toshiba ToPIC95", "Toshiba ToPIC95" },
+    { 0x1217, 0x6836, "O2 Micro 6836", "O2Micro OZ6836" },
+    { 0x1179, 0x0603, "Toshiba ToPIC95-A", "Toshiba ToPIC95-A" },
+    { 0x1179, 0x060a, "Toshiba ToPIC95-B", "Toshiba ToPIC95-B" },
     { 0x1179, 0x060f, "Toshiba ToPIC97", "Toshiba ToPIC97" },
     { 0x119b, 0x1221, "Omega Micro 82C092G", "Omega Micro 82C092G" },
     { 0x8086, 0x1221, "Intel 82092AA", "Intel 82092AA" }
@@ -99,7 +110,7 @@ static int pci_probe(int verbose, int module)
 	while (fgets(s, 256, f) != NULL) {
 	    t = strstr(s, "Device id=");
 	    if (t) {
-		device = strtoul(s+10, NULL, 16);
+		device = strtoul(t+10, NULL, 16);
 		t = strstr(s, "Vendor id=");
 		vendor = strtoul(t+10, NULL, 16);
 		for (i = 0; i < PCI_COUNT; i++)
@@ -142,14 +153,14 @@ static int pci_probe(int verbose, int module)
 static u_char i365_get(u_short sock, u_short reg)
 {
     u_char val = I365_REG(sock, reg);
-    outb_p(val, i365_base); val = inb_p(i365_base+1);
+    outb(val, i365_base); val = inb(i365_base+1);
     return val;
 }
 
 static void i365_set(u_short sock, u_short reg, u_char data)
 {
     u_char val = I365_REG(sock, reg);
-    outb_p(val, i365_base); outb_p(data, i365_base+1);
+    outb(val, i365_base); outb(data, i365_base+1);
 }
 
 static void i365_bset(u_short sock, u_short reg, u_char mask)
@@ -216,8 +227,8 @@ int i365_probe(int verbose, int module)
 	name = "i82365sl DF";
 
     /* Check for Vadem chips */
-    outb_p(0x0e, i365_base);
-    outb_p(0x37, i365_base);
+    outb(0x0e, i365_base);
+    outb(0x37, i365_base);
     i365_bset(0, VG468_MISC, VG468_MISC_VADEMREV);
     val = i365_get(0, I365_IDENT);
     if (val & I365_IDENT_VADEM) {
