@@ -16,7 +16,7 @@
 
 /* Driver author info must always be in the binary.  Version too.. */
 static const char *tc574_version =
-"3c574_cs.c v1.06 4/17/98 Donald Becker/David Hinds, becker@cesdis.gsfc.nasa.gov.\n";
+"3c574_cs.c v1.08 9/24/98 Donald Becker/David Hinds, becker@cesdis.gsfc.nasa.gov.\n";
 
 /*
 				Theory of Operation
@@ -123,7 +123,7 @@ static int irq_list[4] = { -1 };
 #define TX_TIMEOUT  ((800*HZ)/1000)
 
 /* Maximum events (Rx packets, etc.) to handle at each interrupt. */
-static int max_interrupt_work = 10;
+static int max_interrupt_work = 20;
 
 /* Performance features: best left disabled. */
 /* Set to buffer all Tx/RxFIFO accesses. */
@@ -213,14 +213,13 @@ struct el3_private {
 	u16 capabilities, softinfo1, softinfo2;		/* Various, from EEPROM. */
 	u16 advertising;					/* NWay media advertisement */
 	unsigned char phys[2];				/* MII device addresses. */
-	int probe_port;
 	unsigned int
 	  is_full_duplex:1,
 	  force_full_duplex:1,
 	  autoselect:1, default_media:3;	/* Read from the EEPROM/Wn3_Config. */
 };
 
-static char *if_names[] = { "Auto", "10baseT", "10base2", "AUI" };
+static char *if_names[] = { "Auto", "10baseT", "10base2", "AUI", "MII" };
 
 /* Set iff a MII transceiver on any interface requires mdio preamble.
    This only set with the original DP83840 on older 3c905 boards, so the extra
@@ -262,11 +261,7 @@ static int el3_close(struct device *dev);
 #ifdef HAVE_PRIVATE_IOCTL
 static int private_ioctl(struct device *dev, struct ifreq *rq, int cmd);
 #endif
-#ifdef NEW_MULTICAST
 static void set_rx_mode(struct device *dev);
-#else
-static void set_multicast_list(struct device *dev, int num_addrs, void *addrs);
-#endif
 
 static dev_info_t dev_info = "3c574_cs";
 
@@ -457,6 +452,7 @@ static void tc574_config(dev_link_t *link)
 	int last_fn, last_ret, i, j;
 	int ioaddr;
 	u16 *phys_addr;
+	char *cardname;
 
 	handle = link->handle;
 	dev = link->priv;
@@ -541,6 +537,14 @@ static void tc574_config(dev_link_t *link)
 			goto failed;
 		}
 	}
+	tuple.DesiredTuple = CISTPL_VERS_1;
+	if (CardServices(GetFirstTuple, handle, &tuple) == CS_SUCCESS &&
+		CardServices(GetTupleData, handle, &tuple) == CS_SUCCESS &&
+		CardServices(ParseTuple, handle, &tuple, &parse) == CS_SUCCESS) {
+		cardname = parse.version_1.str + parse.version_1.ofs[1];
+	} else
+		cardname = "3Com 3c574";
+
 	/* Extract other info from the EEPROM. */
 	lp->softinfo1 = read_eeprom(ioaddr, 13);
 	lp->softinfo2 = read_eeprom(ioaddr, 15);
@@ -549,19 +553,23 @@ static void tc574_config(dev_link_t *link)
 	if (lp->softinfo1 & 0x8000)
 		lp->force_full_duplex = 1;
 
-	/* The if_port symbol can be set when the module is loaded,
-	   but we always ignore it for now. */
-	if (if_port == 0  ||  if_port == 1  ||  if_port == 4)
-		dev->if_port = if_port;
-	else
-		printk(KERN_NOTICE "3c574_cs: invalid if_port requested\n");
-
-	printk(KERN_INFO "%s: 3Com 3c574, port %#3lx, irq %d, "
-		   "hw_addr ", dev->name, dev->base_addr, dev->irq);
+	printk(KERN_INFO "%s: %s at I/O %#3lx, IRQ %d, "
+		   "MAC Address ", dev->name, cardname, dev->base_addr, dev->irq);
 
 	for (i = 0; i < 6; i++)
 		printk("%02X%s", dev->dev_addr[i], ((i<5) ? ":" : ".\n"));
 
+	/* The if_port symbol can be set when the module is loaded,
+	   but we always ignore it for now. */
+	dev->if_port = 0;
+	if (if_port == 1  ||  if_port == 4) {
+		printk(KERN_INFO "%s: Media override to %s requested.\n",
+			   dev->name, if_names[if_port]);
+		dev->if_port = if_port;
+	} else if (if_port != 0)
+		printk(KERN_NOTICE "%s: Invalid if_port %d requested, ignored.\n",
+			   dev->name, if_port);
+
 	if (dev->mem_start)
 		printk(KERN_INFO"%s:  Acceleration window at memory base %#lx.\n",
 			   dev->name, dev->mem_start);
@@ -589,12 +597,12 @@ static void tc574_config(dev_link_t *link)
 	{
 		int phy, phy_idx = 0;
 		EL3WINDOW(4);
-		for (phy = 0; phy < 32 && phy_idx < sizeof(lp->phys); phy++) {
+		for (phy = 1; phy <= 32 && phy_idx < sizeof(lp->phys); phy++) {
 			int mii_status;
 			mdio_sync(ioaddr, 32);
-			mii_status = mdio_read(ioaddr, phy, 1);
+			mii_status = mdio_read(ioaddr, phy & 0x1f, 1);
 			if (mii_status != 0xffff) {
-				lp->phys[phy_idx++] = phy;
+				lp->phys[phy_idx++] = phy & 0x1f;
 				printk(KERN_INFO "  MII transceiver at index %d, status %x.\n",
 					   phy, mii_status);
 				if ((mii_status & 0x0040) == 0)
@@ -821,14 +829,15 @@ static void tc574_reset(struct device *dev)
 {
 	struct el3_private *lp = (struct el3_private *)dev->priv;
 	int ioaddr = dev->base_addr;
-	int i;
+	int i = 1500;
 
-	outw(RxReset, ioaddr + EL3_CMD);
-	for (i = 500; i >= 0 ; i--)
-		if ( ! inw(ioaddr + EL3_STATUS) & CmdBusy) break;
-	outw(TxReset, ioaddr + EL3_CMD);
-	for (i = 500; i >= 0 ; i--)
-		if ( ! inw(ioaddr + EL3_STATUS) & CmdBusy) break;
+	/* Typically 0 or 1 tick. */
+	outw(TotalReset|0x10, ioaddr + EL3_CMD);
+	while (inw(ioaddr + EL3_STATUS) & CmdBusy)
+		if (--i < 0)  break;
+	if (i < 0)
+		printk(KERN_ERR "%s: Card reset did not complete, status 0x%4.4x.\n",
+			   dev->name, inw(ioaddr + EL3_STATUS));
 
 	/* Set the PIO ctrl bits in the PC card LAN COR using Runner window 1. */
 	if (dev->mem_start || no_wait) {
@@ -903,10 +912,8 @@ static void tc574_reset(struct device *dev)
 
 static int el3_config(struct device *dev, struct ifmap *map)
 {
-	struct el3_private *lp = (struct el3_private *)dev->priv;
 	if ((map->port != (u_char)(-1)) && (map->port != dev->if_port)) {
 		if (map->port <= 3) {
-			lp->probe_port = 0;
 			dev->if_port = map->port;
 			printk(KERN_INFO "%s: switched to %s port\n",
 				   dev->name, if_names[dev->if_port]);
@@ -1154,11 +1161,11 @@ static void el3_interrupt IRQ(int irq, void *dev_id, struct pt_regs *regs)
 				if (fifo_diag & 0x0400) {
 					outw(TxReset, ioaddr + EL3_CMD);
 					for (i = 500; i >= 0 ; i--)
-						if ( ! inw(ioaddr + EL3_STATUS) & CmdBusy) break;
+						if ((inw(ioaddr + EL3_STATUS) & CmdBusy) == 0) break;
 				}
 				outw(RxReset, ioaddr + EL3_CMD);
 				for (i = 500; i >= 0 ; i--)
-					if ( ! inw(ioaddr + EL3_STATUS) & CmdBusy) break;
+					if ((inw(ioaddr + EL3_STATUS) & CmdBusy) == 0) break;
 				set_rx_mode(dev);
 				outw(RxEnable, ioaddr + EL3_CMD);
 				outw(TxEnable, ioaddr + EL3_CMD);
@@ -1262,7 +1269,7 @@ static int el3_rx(struct device *dev, int worklimit)
 			short pkt_len = rx_status & 0x7ff;
 			struct sk_buff *skb;
 
-			skb = ALLOC_SKB(pkt_len+3);
+			skb = dev_alloc_skb(pkt_len+5);
 
 			DEBUG(3, "    Receiving packet size %d status %4.4x.\n",
 				  pkt_len, rx_status);
@@ -1278,14 +1285,15 @@ static int el3_rx(struct device *dev, int worklimit)
 				printk("%s: Start Rx %d -- RunnerRdCtrl is %4.4x.\n",
 					   dev->name, pkt_len, inw(ioaddr + RunnerRdCtrl));
 #endif
+				skb_reserve(skb, 2);
 				if (dev->mem_start) {
-#define BLOCK_INPUT(buf, len) memcpy((void*)dev->mem_start, buf, (len+3)&~3);
-					GET_PACKET(dev, skb, pkt_len);
+					memcpy((void*)dev->mem_start,
+						   skb_put(skb, pkt_len), (pkt_len+3)&~3);
 				} else {
-#undef BLOCK_INPUT
-#define BLOCK_INPUT(buf, len) insl(ioaddr+RX_FIFO, buf, (len+3)>>2)
-					GET_PACKET(dev, skb, pkt_len);
+					insl(ioaddr+RX_FIFO, skb_put(skb, pkt_len),
+						 (pkt_len+3)>>2);
 				}
+				skb->protocol = eth_type_trans(skb, dev);
 
 #if 0
 				printk("%s: RunnerRdCtrl is now %4.4x.\n",
@@ -1308,7 +1316,7 @@ static int el3_rx(struct device *dev, int worklimit)
 		/* Wait a limited time to skip this packet. */
 		{
 			int i = 200;
-			while (--i >= 0  &&  inw(ioaddr + EL3_STATUS) & CmdBusy)
+			while (--i >= 0  &&  (inw(ioaddr + EL3_STATUS) & CmdBusy))
 				;
 		}
 	}
@@ -1376,11 +1384,7 @@ static int private_ioctl(struct device *dev, struct ifreq *rq, int cmd)
    typical PC card machines, so we omit the message.
    */
 
-#ifdef NEW_MULTICAST
 static void set_rx_mode(struct device *dev)
-#else
-static void set_rx_mode(struct device *dev, int num_addrs, void *addrs)
-#endif
 {
 	int ioaddr = dev->base_addr;
 
