@@ -3,8 +3,8 @@
     A PCMCIA ethernet driver for SMC91c92-based cards.
 
     This driver supports Megahertz PCMCIA ethernet cards; and
-    Megahertz, Motorola, and Ositech ethernet/modem multifunction
-    cards.
+    Megahertz, Motorola, Ositech, and Psion Dacom ethernet/modem
+    multifunction cards.
 
     Copyright (C) 1998 David A. Hinds -- dhinds@hyper.stanford.edu
 
@@ -91,7 +91,7 @@ MODULE_PARM(irq_list, "1-4i");
 #define USE_32_BIT		1
 
 /* Time in jiffies before concluding Tx hung */
-#define TX_TIMEOUT		40
+#define TX_TIMEOUT		((400*HZ)/1000)
 
 /* Maximum events (Rx packets, etc.) to handle at each interrupt. */
 #define INTR_WORK		4
@@ -324,11 +324,7 @@ static dev_link_t *smc91c92_attach(void)
     dev->hard_start_xmit = &smc_start_xmit;
     dev->get_stats = &smc91c92_get_stats;
     dev->set_config = &s9k_config;
-#ifdef NEW_MULTICAST
     dev->set_multicast_list = &set_rx_mode;
-#else
-#warning "This driver does not support multicast with old kernels."
-#endif
     ether_setup(dev);
     dev->name = ((struct smc_private *)dev->priv)->node.dev_name;
     dev->init = &smc91c92_init;
@@ -728,7 +724,7 @@ static int osi_config(dev_link_t *link)
     return i;
 }
 
-static int osi_setup(dev_link_t *link)
+static int osi_setup(dev_link_t *link, u_short manfid)
 {
     client_handle_t handle = link->handle;
     struct device *dev = link->priv;
@@ -736,7 +732,7 @@ static int osi_setup(dev_link_t *link)
     u_char buf[255];
     int i;
     
-    tuple.Attributes = 0;
+    tuple.Attributes = TUPLE_RETURN_COMMON;
     tuple.TupleData = buf;
     tuple.TupleDataMax = sizeof(buf);
     tuple.TupleOffset = 0;
@@ -755,6 +751,8 @@ static int osi_setup(dev_link_t *link)
     for (i = 0; i < 6; i++)
 	dev->dev_addr[i] = buf[i+2];
 
+    if (manfid != MANFID_OSITECH) return 0;
+    
     /* Make sure both functions are powered up */
     set_bits(0x300, link->io.BasePort1 + OSITECH_AUI_PWR);
     /* Now, turn on the interrupt for both card functions */
@@ -844,6 +842,7 @@ static void smc91c92_config(dev_link_t *link)
     link->conf.Present = parse.config.rmask[0];
     
     tuple.DesiredTuple = CISTPL_MANFID;
+    tuple.Attributes = TUPLE_RETURN_COMMON;
     i = first_tuple(handle, &tuple, &parse);
     CS_EXIT_TEST(i, GetFirstTuple, config_failed);
     lp->manfid = le16_to_cpu(buf[0]);
@@ -890,7 +889,8 @@ static void smc91c92_config(dev_link_t *link)
 
     switch (lp->manfid) {
     case MANFID_OSITECH:
-	i = osi_setup(link); break;
+    case MANFID_PSION:
+	i = osi_setup(link, lp->manfid); break;
     case MANFID_SMC:
     case MANFID_NEW_MEDIA:
 	i = smc_setup(link); break;
@@ -1565,7 +1565,7 @@ static void smc_rx(struct device *dev)
 	struct sk_buff *skb;
 	
 	/* Note: packet_length adds 5 or 6 extra bytes here! */
-	skb = ALLOC_SKB(packet_length);
+	skb = dev_alloc_skb(packet_length+2);
 	
 	if ( skb == NULL ) {
 	    DEBUG(1, "%s: Low memory, packet dropped.\n", dev->name);
@@ -1575,9 +1575,10 @@ static void smc_rx(struct device *dev)
 	}
 	
 	packet_length -= (rx_status & RS_ODDFRAME ? 5 : 6);
-
-#define BLOCK_INPUT(buf, len) insw_ns(ioaddr+DATA_1, buf, (len+1)>>1)
-	GET_PACKET(dev, skb, packet_length);
+	skb_reserve(skb, 2);
+	insw_ns(ioaddr+DATA_1, skb_put(skb, packet_length),
+		(packet_length+1)>>1);
+	skb->protocol = eth_type_trans(skb, dev);
 	
 	skb->dev = dev;
 	netif_rx(skb);
