@@ -1,11 +1,29 @@
 /*======================================================================
 
-    PCMCIA card CIS dump
+    PCMCIA card configuration register dump
 
-    Written by David Hinds, dhinds@allegro.stanford.edu
+    dump_cisreg.c 1.18 1998/08/01 13:25:40
+
+    The contents of this file are subject to the Mozilla Public
+    License Version 1.0 (the "License"); you may not use this file
+    except in compliance with the License. You may obtain a copy of
+    the License at http://www.mozilla.org/MPL/
+
+    Software distributed under the License is distributed on an "AS
+    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
+    implied. See the License for the specific language governing
+    rights and limitations under the License.
+
+    The initial developer of the original code is David A. Hinds
+    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
+    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
     
 ======================================================================*/
 
+#ifndef __linux__
+#include <pcmcia/u_compat.h>
+#endif
+
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -23,11 +41,12 @@
 #include <pcmcia/cisreg.h>
 #include <pcmcia/ds.h>
 
-static const char *version =
-"dump_cisreg.c 1.11 1997/12/29 17:48:45 (David Hinds)\n";
-
 /*====================================================================*/
 
+#ifdef __linux__
+
+static int major = 0;
+
 static int lookup_dev(char *name)
 {
     FILE *f;
@@ -47,15 +66,18 @@ static int lookup_dev(char *name)
 	return n;
     else
 	return -1;
-} /* lookup_dev */
+}
+
+#endif /* __linux__ */
 
 /*====================================================================*/
 
-static int open_dev(dev_t dev)
+static int open_sock(int sock)
 {
-    char *fn;
+#ifdef __linux__
     int fd;
-    
+    char *fn;
+    dev_t dev = (major<<8) + sock;
     if ((fn = tmpnam(NULL)) == NULL)
 	return -1;
     if (mknod(fn, (S_IFCHR|S_IREAD|S_IWRITE), dev) != 0)
@@ -63,7 +85,13 @@ static int open_dev(dev_t dev)
     fd = open(fn, O_RDONLY);
     unlink(fn);
     return fd;
-} /* open_dev */
+#endif
+#ifdef __BEOS__
+    char fn[B_OS_NAME_LENGTH];
+    sprintf(fn, "/dev/pcmcia/sock%d", sock);
+    return open(fn, O_RDONLY);
+#endif
+} /* open_sock */
 
 /*====================================================================*/
 
@@ -191,21 +219,20 @@ static void dump_all(int fd, int fn, int mfc, u_int mask)
 
 int main(int argc, char *argv[])
 {
-    int i, j, nfn, major, fd, ret;
+    int i, j, nfn, fd, ret;
     u_int mask;
     ds_ioctl_arg_t arg;
 
-    if ((argc > 1) && (strcmp(argv[1], "-v") == 0))
-	printf("%s", version);
-    
+#ifdef __linux__
     major = lookup_dev("pcmcia");
     if (major < 0) {
 	fprintf(stderr, "no pcmcia driver in /proc/devices\n");
 	exit(EXIT_FAILURE);
     }
+#endif
     
     for (i = 0; i < MAX_SOCKS; i++) {
-	fd = open_dev((major<<8)+i);
+	fd = open_sock(i);
 	if (fd < 0) break;
 	
 	arg.tuple.TupleDataMax = sizeof(arg.tuple_parse.data);
@@ -220,7 +247,12 @@ int main(int argc, char *argv[])
 	    nfn = 1;
 	    arg.tuple.DesiredTuple = CISTPL_DEVICE;
 	    ret = ioctl(fd, DS_GET_FIRST_TUPLE, &arg);
+	    if (ret != 0) {
+		if (errno != ENODEV) perror("ioctl()");
+		continue;
+	    }
 	}
+
 	arg.tuple.DesiredTuple = CISTPL_CONFIG;
 	
 	for (j = 0; j < nfn; j++) {
