@@ -2,7 +2,7 @@
 
     PCMCIA Card Services -- core services
 
-    cs.c 1.197 1998/08/20 23:06:31
+    cs.c 1.213 1999/02/13 04:50:18
     
     The contents of this file are subject to the Mozilla Public
     License Version 1.0 (the "License"); you may not use this file
@@ -59,11 +59,14 @@ static int handle_apm_event(apm_event_t event);
 int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 static const char *version =
-"cs.c 1.197 1998/08/20 23:06:31 (David Hinds)";
+"cs.c 1.213 1999/02/13 04:50:18 (David Hinds)";
 #endif
 
-static const char *release = "Linux PCMCIA Card Services " CS_RELEASE;
+#ifdef __BEOS__
+static const char *release = "BeOS PCMCIA Card Services " CS_RELEASE;
+#endif
 #ifdef __LINUX__
+static const char *release = "Linux PCMCIA Card Services " CS_RELEASE;
 static const char *kernel = "kernel build: " UTS_RELEASE " " UTS_VERSION;
 #endif
 static const char *options = "options: "
@@ -172,10 +175,12 @@ static const lookup_t error_table[] = {
 
 static const lookup_t service_table[] = {
     { AccessConfigurationRegister,	"AccessConfigurationRegister" },
+    { AddSocketServices,		"AddSocketServices" },
     { AdjustResourceInfo,		"AdjustResourceInfo" },
     { CheckEraseQueue,			"CheckEraseQueue" },
     { CloseMemory,			"CloseMemory" },
     { DeregisterClient,			"DeregisterClient" },
+    { DeregisterEraseQueue,		"DeregisterEraseQueue" },
     { GetCardServicesInfo,		"GetCardServicesInfo" },
     { GetClientInfo,			"GetClientInfo" },
     { GetConfigurationInfo,		"GetConfigurationInfo" },
@@ -195,6 +200,7 @@ static const lookup_t service_table[] = {
     { ParseTuple,			"ParseTuple" },
     { ReadMemory,			"ReadMemory" },
     { RegisterClient,			"RegisterClient" },
+    { RegisterEraseQueue,		"RegisterEraseQueue" },
     { RegisterMTD,			"RegisterMTD" },
     { ReleaseConfiguration,		"ReleaseConfiguration" },
     { ReleaseIO,			"ReleaseIO" },
@@ -213,7 +219,10 @@ static const lookup_t service_table[] = {
     { BindMTD,				"BindMTD" },
     { ReportError,			"ReportError" },
     { SuspendCard,			"SuspendCard" },
-    { ResumeCard,			"ResumeCard" }
+    { ResumeCard,			"ResumeCard" },
+    { EjectCard,			"EjectCard" },
+    { InsertCard,			"InsertCard" },
+    { ReplaceCIS,			"ReplaceCIS" }
 };
 #define SERVICE_COUNT (sizeof(service_table)/sizeof(lookup_t))
 
@@ -236,7 +245,7 @@ static void init_socket(socket_info_t *s)
 	io.map = i;
 	s->ss_entry(s->sock, SS_SetIOMap, &io);
     }
-    for (i = 0; i < 4; i++) {
+    for (i = 0; i < 5; i++) {
 	mem.map = i;
 	s->ss_entry(s->sock, SS_SetMemMap, &mem);
     }
@@ -302,7 +311,7 @@ int register_ss_entry(int nsock, ss_entry_t ss_entry)
 	init_socket(s);
 	ss_entry(ns, SS_InquireSocket, &s->cap);
 #ifdef HAS_PROC_BUS
-	{
+	if (proc_pccard) {
 	    char name[3];
 #ifdef PCMCIA_DEBUG
 	    struct proc_dir_entry *ent;
@@ -339,12 +348,14 @@ void unregister_ss_entry(ss_entry_t ss_entry)
 	    break;
 	} else {
 #ifdef HAS_PROC_BUS
-	    char name[3];
-	    sprintf(name, "%02d", i);
+	    if (proc_pccard) {
+		char name[3];
+		sprintf(name, "%02d", i);
 #ifdef PCMCIA_DEBUG
-	    remove_proc_entry("clients", s->proc);
+		remove_proc_entry("clients", s->proc);
 #endif
-	    remove_proc_entry(name, proc_pccard);
+		remove_proc_entry(name, proc_pccard);
+	    }
 #endif
 	    while (s->clients) {
 		client = s->clients;
@@ -439,17 +450,20 @@ static void setup_socket(u_long i)
 	s->state |= SOCKET_PRESENT;
 	s->socket.flags = 0;
 	if (val & SS_3VCARD)
-	    s->socket.Vcc = 33;
+	    s->socket.Vcc = s->socket.Vpp = 33;
 	else if (!(val & SS_XVCARD))
-	    s->socket.Vcc = 50;
+	    s->socket.Vcc = s->socket.Vpp = 50;
 	else {
 	    printk(KERN_NOTICE "cs: socket %ld: unsupported "
 		   "voltage key\n", i);
 	    s->socket.Vcc = 0;
 	}
-#ifdef CONFIG_CARDBUS
-	if (val & SS_CARDBUS) s->state |= SOCKET_CARDBUS;
+	if (val & SS_CARDBUS) {
+	    s->state |= SOCKET_CARDBUS;
+#ifndef CONFIG_CARDBUS
+	    printk(KERN_NOTICE "cs: unsupported card type detected!\n");
 #endif
+	}
 	s->ss_entry(s->sock, SS_SetSocket, &s->socket);
 	s->setup.function = &reset_socket;
 	s->setup.expires = RUN_AT(vcc_settle);
@@ -656,7 +670,7 @@ static int handle_apm_event(apm_event_t event)
 	for (i = 0; i < sockets; i++) {
 	    s = socket_table[i];
 	    /* Do this just to reinitialize the socket */
-	    s->ss_entry(s->sock, SS_SetSocket, &dead_socket);
+	    init_socket(s);
 	    s->ss_entry(s->sock, SS_GetStatus, &stat);
 	    /* If there was or is a card here, we need to do something
 	       about it... but parse_events will sort it all out. */
@@ -916,14 +930,15 @@ static int get_configuration_info(client_handle_t handle,
     if (!(s->state & SOCKET_PRESENT))
 	return CS_NO_CARD;
 
+    if (handle->Function == BIND_FN_ALL) {
+	if (config->Function && (config->Function >= s->functions))
+	    return CS_BAD_ARGS;
+    } else
+	config->Function = handle->Function;
+    
 #ifdef CONFIG_CARDBUS
     if (s->state & SOCKET_CARDBUS) {
 	u_char fn = config->Function;
-	if (handle->Function == BIND_FN_ALL) {
-	    if (fn && (fn >= s->functions))
-		return CS_BAD_ARGS;
-	} else
-	    fn = handle->Function;
 	memset(config, 0, sizeof(config_info_t));
 	config->Function = fn;
 	config->Vcc = s->socket.Vcc;
@@ -933,6 +948,8 @@ static int get_configuration_info(client_handle_t handle,
 	    config->Attributes = CONF_VALID_CLIENT;
 	    config->IntType = INT_CARDBUS;
 	    config->AssignedIRQ = s->irq.AssignedIRQ;
+	    if (config->AssignedIRQ)
+		config->Attributes |= CONF_ENABLE_IRQ;
 	    config->BasePort1 = s->io[0].BasePort;
 	    config->NumPorts1 = s->io[0].NumPorts;
 	}
@@ -940,12 +957,6 @@ static int get_configuration_info(client_handle_t handle,
     }
 #endif
     
-    if (handle->Function == BIND_FN_ALL) {
-	if (config->Function && (config->Function >= s->functions))
-	    return CS_BAD_ARGS;
-    } else {
-	config->Function = handle->Function;
-    }
     c = (s->config != NULL) ? &s->config[config->Function] : NULL;
     
     if ((c == NULL) || !(c->state & CONFIG_LOCKED)) {
@@ -1033,8 +1044,7 @@ static int get_next_client(client_handle_t *handle, client_req_t *req)
 /*======================================================================
 
     Get the current socket state bits.  We don't support the latched
-    SocketState yet.  We also don't support the Pin Replacement
-    Register.
+    SocketState yet: I haven't seen any point for it.
     
 ======================================================================*/
 
@@ -1050,9 +1060,7 @@ static int get_status(client_handle_t handle, cs_status_t *status)
     s->ss_entry(s->sock, SS_GetStatus, &val);
     status->CardState = status->SocketState = 0;
     status->CardState |= (val & SS_DETECT) ? CS_EVENT_CARD_DETECT : 0;
-#ifdef CONFIG_CARDBUS
     status->CardState |= (val & SS_CARDBUS) ? CS_EVENT_CB_DETECT : 0;
-#endif
     status->CardState |= (val & SS_3VCARD) ? CS_EVENT_3VCARD : 0;
     status->CardState |= (val & SS_XVCARD) ? CS_EVENT_XVCARD : 0;
     if (s->state & SOCKET_SUSPEND)
@@ -1254,10 +1262,8 @@ static int register_client(client_handle_t *handle, client_reg_t *req)
     client->event_callback_args = req->event_callback_args;
     client->event_callback_args.client_handle = client;
 
-#ifdef CONFIG_CARDBUS
     if (s->state & SOCKET_CARDBUS)
 	client->state |= CLIENT_CARDBUS;
-#endif
     
     if ((!(s->state & SOCKET_CARDBUS)) && (s->functions == 0) &&
 	(client->Function != BIND_FN_ALL)) {
@@ -1382,7 +1388,7 @@ static int release_io(client_handle_t handle, io_req_t *req)
 
 /*====================================================================*/
 
-static int release_irq(client_handle_t handle, irq_req_t *req)
+static int cs_release_irq(client_handle_t handle, irq_req_t *req)
 {
     socket_info_t *s;
     if (CHECK_HANDLE(handle) || !(handle->state & CLIENT_IRQ_REQ))
@@ -1411,7 +1417,8 @@ static int release_irq(client_handle_t handle, irq_req_t *req)
 	    IRQ_MAP(req->AssignedIRQ, NULL);
 #endif
 #ifdef __BEOS__
-	remove_io_interrupt_handler(req->AssignedIRQ, req->Handler);
+	remove_io_interrupt_handler(req->AssignedIRQ, req->Handler,
+				    req->Instance);
 #endif
     }
 
@@ -1421,7 +1428,7 @@ static int release_irq(client_handle_t handle, irq_req_t *req)
 #endif
     
     return CS_SUCCESS;
-} /* release_irq */
+} /* cs_release_irq */
 
 /*====================================================================*/
 
@@ -1468,6 +1475,8 @@ static int request_configuration(client_handle_t handle,
     
 #ifdef CONFIG_CARDBUS
     if (handle->state & CLIENT_CARDBUS) {
+	if (!(req->IntType & INT_CARDBUS))
+	    return CS_UNSUPPORTED_MODE;
 	if (s->lock_count != 0)
 	    return CS_CONFIGURATION_LOCKED;
 	cb_enable(s);
@@ -1477,6 +1486,8 @@ static int request_configuration(client_handle_t handle,
     }
 #endif
     
+    if (req->IntType & INT_CARDBUS)
+	return CS_UNSUPPORTED_MODE;
     c = CONFIG(handle);
     if (c->state & CONFIG_LOCKED)
 	return CS_CONFIGURATION_LOCKED;
@@ -1598,14 +1609,16 @@ static int request_io(client_handle_t handle, io_req_t *req)
     if (!(s->state & SOCKET_PRESENT))
 	return CS_NO_CARD;
 
-#ifdef CONFIG_CARDBUS
     if (handle->state & CLIENT_CARDBUS) {
+#ifdef CONFIG_CARDBUS
 	int ret = cb_config(s);
 	if (ret == CS_SUCCESS)
 	    handle->state |= CLIENT_IO_REQ;
 	return ret;
-    }
+#else
+	return CS_UNSUPPORTED_FUNCTION;
 #endif
+    }
     
     c = CONFIG(handle);
     if (c->state & CONFIG_LOCKED)
@@ -1681,8 +1694,9 @@ static int cs_request_irq(client_handle_t handle, irq_req_t *req)
 	ret = CS_IN_USE;
 	if (req->IRQInfo1 & IRQ_INFO2_VALID) {
 	    mask = req->IRQInfo2 & s->cap.irq_mask;
+	    mask &= ~(1 << s->cap.pci_irq);
 	    for (try = 0; try < 2; try++) {
-		for (irq = 0; irq < NR_IRQS; irq++)
+		for (irq = 0; irq < 32; irq++)
 		    if ((mask >> irq) & 1) {
 			ret = try_irq(req->Attributes, irq, try);
 			if (ret == 0) break;
@@ -1762,7 +1776,8 @@ static int request_window(client_handle_t *handle, win_req_t *req)
     win->base = req->Base;
     win->size = req->Size;
     if (find_mem_region(&win->base, win->size, (*handle)->dev_info,
-			(s->cap.cardbus == 0)))
+			(req->Attributes & WIN_MAP_BELOW_1MB) ||
+			!(s->cap.features & SS_HAS_PAGE_REGS)))
 	return CS_IN_USE;
     req->Base = win->base;
     (*handle)->state |= CLIENT_WIN_REQ(w);
@@ -2068,7 +2083,7 @@ int CardServices(int func, void *a1, void *a2, void *a3)
     case ReleaseIO:
 	return release_io(a1, a2); break;
     case ReleaseIRQ:
-	return release_irq(a1, a2); break;
+	return cs_release_irq(a1, a2); break;
     case ReleaseWindow:
 	return release_window(a1); break;
     case RequestConfiguration:
@@ -2122,10 +2137,8 @@ int CardServices(int func, void *a1, void *a2, void *a3)
 #undef CONFIG_MODVERSIONS
 static struct symbol_table cs_symtab = {
 #include <linux/symtab_begin.h>
-#if (LINUX_VERSION_CODE >= VERSION(1,3,0))
 #undef X
 #define X(sym) { (void *)&sym, SYMBOL_NAME_STR(sym) }
-#endif
     X(register_ss_entry),
     X(unregister_ss_entry),
     X(CardServices),
@@ -2159,7 +2172,7 @@ int init_module(void)
     register_symtab(&cs_symtab);
 #ifdef HAS_PROC_BUS
     proc_pccard = create_proc_entry("pccard", S_IFDIR, proc_bus);
-    {
+    if (proc_pccard) {
 	struct proc_dir_entry *ent;
 	ent = create_proc_entry("memory", 0, proc_pccard);
 	ent->read_proc = proc_read_mem;
@@ -2172,8 +2185,10 @@ void cleanup_module(void)
 {
     printk(KERN_INFO "unloading PCMCIA Card Services\n");
 #ifdef HAS_PROC_BUS
-    remove_proc_entry("memory", proc_pccard);
-    remove_proc_entry("pccard", proc_bus);
+    if (proc_pccard) {
+	remove_proc_entry("memory", proc_pccard);
+	remove_proc_entry("pccard", proc_bus);
+    }
 #endif
 #ifdef CONFIG_APM
     apm_unregister_callback(&handle_apm_event);
@@ -2189,7 +2204,9 @@ void cleanup_module(void)
 
 isa_module_info *isa = NULL;
 pci_module_info *pci = NULL;
+config_manager_for_bus_module_info *cm = NULL;
 module_info *i82365 = NULL;
+typedef struct module_info mod_t;
 
 static status_t std_ops(int32 op)
 {
@@ -2199,18 +2216,21 @@ static status_t std_ops(int32 op)
 	printk(KERN_INFO "  %s\n", options);
 	DEBUG(0, ("%s\n", version));
 	init_timer();
-	if (get_module(B_ISA_MODULE_NAME, (struct module_info **)&isa)
-	    != B_OK)
+	if (get_module(B_ISA_MODULE_NAME, (mod_t **)&isa) != B_OK)
 	    return B_ERROR;
-	if (get_module(B_PCI_MODULE_NAME, (struct module_info **)&pci)
-	    != B_OK)
+	if (get_module(B_PCI_MODULE_NAME, (mod_t **)&pci) != B_OK)
 	    return B_ERROR;
-	get_module("busses/pcmcia/i82365", &i82365);
+	if (get_module(B_CONFIG_MANAGER_FOR_BUS_MODULE_NAME,
+		       (mod_t **)&cm) != B_OK)
+	    return B_ERROR;
+	get_module(SS_MODULE_NAME("i82365"), &i82365);
 	break;
     case B_MODULE_UNINIT:
 	printk(KERN_INFO "unloading PCMCIA Card Services\n");
-	if (i82365 != NULL) put_module("busses/pcmcia/i82365");
+	if (i82365 != NULL) put_module(SS_MODULE_NAME("i82365"));
 	release_resource_db();
+	if (cm != NULL)
+	    put_module(B_CONFIG_MANAGER_FOR_BUS_MODULE_NAME);
 	if (pci != NULL) put_module(B_PCI_MODULE_NAME);
 	if (isa != NULL) put_module(B_ISA_MODULE_NAME);
 	stop_timer();
@@ -2226,8 +2246,8 @@ static status_t no_ops(int32 op)
 
 static cs_client_module_info cs_client_info = {
     { { CS_CLIENT_MODULE_NAME, B_KEEP_LOADED, &std_ops }, NULL },
-    &CardServices,
-    &MTDHelperEntry,
+    (int (*)(int, ...))&CardServices,
+    (int (*)(int, ...))&MTDHelperEntry,
     &add_timer,
     &del_timer
 };
@@ -2237,7 +2257,10 @@ static cs_socket_module_info cs_socket_info = {
     &register_ss_entry,
     &unregister_ss_entry,
     &add_timer,
-    &del_timer
+    &del_timer,
+    &register_resource,
+    &release_resource,
+    &check_resource
 };
 
 _EXPORT module_info *modules[] = {
