@@ -2,7 +2,7 @@
 
     PC Card Driver Services
     
-    ds.c 1.86 1998/07/28 09:22:30
+    ds.c 1.90 1999/02/11 06:17:24
     
     The contents of this file are subject to the Mozilla Public
     License Version 1.0 (the "License"); you may not use this file
@@ -49,7 +49,7 @@ int pc_debug = PCMCIA_DEBUG;
 MODULE_PARM(pc_debug, "i");
 #define DEBUG(n, args...) if (pc_debug>(n)) printk(KERN_DEBUG args)
 static const char *version =
-"ds.c 1.86 1998/07/28 09:22:30 (David Hinds)";
+"ds.c 1.90 1999/02/11 06:17:24 (David Hinds)";
 #else
 #define DEBUG(n, args...)
 #endif
@@ -474,14 +474,14 @@ static int ds_open(struct inode *inode, struct file *file)
     if ((i >= sockets) || (sockets == 0))
 	return -ENODEV;
     s = &socket_table[i];
-    MOD_INC_USE_COUNT;
-    
     if ((file->f_flags & O_ACCMODE) != O_RDONLY) {
 	if (s->state & SOCKET_BUSY)
 	    return -EBUSY;
 	else
 	    s->state |= SOCKET_BUSY;
     }
+    
+    MOD_INC_USE_COUNT;
     user = kmalloc(sizeof(user_info_t), GFP_KERNEL);
     user->event_tail = user->event_head = 0;
     user->next = s->user;
@@ -808,6 +808,7 @@ static struct file_operations ds_fops = {
     ds_ioctl,		/* ioctl */
     NULL,		/* mmap */
     ds_open,		/* open */
+    NULL_FLUSH		/* flush */
     ds_release,		/* release */
     NULL		/* fsync */
 };
@@ -817,10 +818,8 @@ static struct file_operations ds_fops = {
 #undef CONFIG_MODVERSIONS
 static struct symbol_table ds_symtab = {
 #include <linux/symtab_begin.h>
-#if (LINUX_VERSION_CODE >= VERSION(1,3,0))
 #undef X
 #define X(sym) { (void *)&sym, SYMBOL_NAME_STR(sym) }
-#endif
     X(register_pccard_driver),
     X(unregister_pccard_driver),
 #include <linux/symtab_end.h>
@@ -851,6 +850,10 @@ int init_module(void)
 	printk(KERN_NOTICE "ds: Card Services release does not match!\n");
 	return -1;
     }
+    if (serv.Count == 0) {
+	printk(KERN_NOTICE "ds: no socket drivers loaded!\n");
+	return -1;
+    }
     
     sockets = serv.Count;
     socket_table = kmalloc(sockets*sizeof(socket_info_t), GFP_KERNEL);
